#! /usr/bin/env python

# Analyse des fichiers journaux
# /var/log/samba/userlogons  et /var/squid/logs/access.log

import string
from time import time, localtime
import os           # module pour les appels au systme d'exploitation
import pwd                       # accs aux infos de /etc/passwd
from os.path import exists

version = '27/09/2001'
samba_log_path = "/var/log/samba/userlogons"
squid_log_path = "/var/squid/logs/access.log"
smbDico ={}                      # Dico pour les donnes de Samba
hstDico={}                       # Dico pour la rsolution des adresses IP

def hotes_connus():
   "Construction d'un dictionnaire pour la rsolution des adresses IP"
   if exists("/etc/hosts") ==0:
      return
   file = open("/etc/hosts","r")
   while 1:
      li = file.readline()
      if len(li) ==0:              # dtection fin de fichier
         break
      tl = string.split(li)        # liste intermdiaire
      # on ne garde que les lignes contenant qqc :
      if len(tl) <2:
         continue
      # on ne garde que les lignes commanant par une adresse IP :
      if string.count(tl[0],'.') !=3:
         continue
      # Le nom du poste est extrait du second item de la liste :
      p = tl[1]
      # On en spare le nom du poste du nom de domaine ventuel :
      p = string.split(p,'.')
      # Ajout du nom au dictionnaire. L'adresse IP servira d'index :
      hstDico[tl[0]] = p[0]
   file.close()

def log2smbDico():
   "Conversion du fichier journal de Samba en un dictionnaire Python"
   # On va utiliser un dictionnaire. Les cls de celui-ci seront les noms
   # des machines qui se sont connectes. Pour chacune d'elles, on tablit
   # une liste de tuples (utilisateur,date de connection)
   file = open(samba_log_path,"r")
   while 1:
      li = file.readline()
      if len(li) < 3:              # dtection fin de fichier
         break
      tl = string.split(li)        # liste intermdiaire
      poste = string.upper(tl[0])
      user = tl[1]
      date = float(tl[2])
      datus = smbDico.get(poste,[])   # liste des tuples (user,date) pour ce poste
      datus.append((user,date))
      smbDico[poste] = datus
   file.close()

def ut_date(lut):
   "afficher la liste <lut> qui contient des tuples (utilisateur,date)"
   for i in lut:
      print i[0],                      # identifiant de l'utilisateur
      try:
         tu = pwd.getpwnam(i[0])       # retourne un tuple avec paramtres user
         # toilettage du nom complet (remplacement des souligns) :
         comm = string.replace(string.strip(tu[4]),"_"," ")
         print "(%s)" % (comm),        # nom complet de l'utilisateur
      except:
         print "(* Inconnu dans /etc/passwd *)",
      t = localtime(float(i[1]))
      print "%s/%s/%s  %s:%s:%s" % (t[2],t[1],t[0],t[3],t[4],t[5])

def suspect(poste,date):
   "retourne l'utilisateur prsum sur ce poste et  cette date"
   ut =''
   da =0.0
   if smbDico.has_key(poste):
      lusdat = smbDico.get(poste,[])   # liste des tuples (user,date) pour ce poste
      for i in lusdat:
         if i[1] > date:
            break
         ut = i[0]
         da = i[1]
   return (ut,da)

def ut_par_poste():
   "Listing des utilisateurs pour un poste de travail"
   print 'Entrez le nom du poste : ',
   pt = string.upper(raw_input())
   if smbDico.has_key(pt):
      print "*** Poste %s ***" % (pt)
      ut_date(smbDico[pt])       # -> afficher la liste des tuples user,date
   else:
      print "Aucune rfrence  ce poste dans le fichier journal de Samba."

def ut_tous_postes():
   "Listing de tous les utilisateurs de tous les postes"
   # transformation du dico en une liste de tuples :
   dico_tr = smbDico.items()
   dico_tr.sort()             # tri
   for p in dico_tr:
      print "##### Poste %s #####" % (p[0])
      ut_date(p[1])           # -> afficher la liste des tuples user,date
      print

def postes_par_ut():
   "Listing des postes utiliss par une personne donne"
   log2smbDico()
   print "Entrez l'identifiant de cette personne : ",
   user = raw_input()
   print
   # transformation du dico en une liste de tuples :
   dico_tr = smbDico.items()
   lipo =[]
   for p in dico_tr:
      for i in p[1]:
         if string.upper(i[0]) == string.upper(user):
            lipo.append((p[0], i[1]))
   if len(lipo) >0:
      print "Postes utiliss par cette personne :"
      for d in lipo:
         print d[0],                # Poste utilis
         t = localtime(d[1])        # dcompose la date systme en un tuple
         print "%s/%s/%s  %s:%s:%s" % (t[2],t[1],t[0],t[3],t[4],t[5])
   else:
      print "Aucune rfrence  cet utilisateur dans le fichier journal de Samba."

def postes_par_url():
   "Postes ayant consult une URL particulire"
   print "Entrez la chane de caractres  rechercher dans les URL : ",
   keyw = raw_input()
   print
   # Pour rechercher une chane dans l'ensemble du fichier de logs, on va se
   # servir de l'utilitaire <grep> bien connu dans le monde Unix.
   # Pour rcuprer la sortie de grep, on peut ouvrir un 'pipe'  l'aide de
   # la fonction Python : os.popen("commande"). Cette fonction retourne un
   # "objet fichier" reli au pipe, auquel on peut appliquer les mthodes
   # applicables aux fichiers, telles write() ou readline() :
   f = os.popen("grep -ci %s %s" % (keyw, squid_log_path), 'r')
   # Dans ce cas ci, l'objet fichier ne contient qu'une ligne (un nombre) :
   count = string.atoi(f.readline())
   print "Il y a %s occurrences de la chane %s dans le journal de Squid." % (count, keyw)
   if count ==0:
      return
   print "Voulez-vous lister les postes de travail concerns ? (o/n) ",
   ch = string.upper(raw_input())
   print
   if ch[0] =='N':
      return
   # On fait  nouveau appel  <grep>, dont la sortie est cette fois
   # directement transforme en une liste de lignes :
   lignes = os.popen("grep -i %s %s" % (keyw, squid_log_path), 'r').readlines()
   # Chaque ligne est "splitte" en une liste <items> :
   for r in lignes:
      items = string.split(r)
      d = float(items[0])      # l'item 0 est la date d'accs
      t = localtime(d)         # dcompose la date systme en un tuple
      # l'item 2 est l'adresse IP du poste. On cherche le nom corresp. :
      if hstDico.has_key(items[2]):
         poste = string.upper(hstDico[items[2]])
         # rechercher l'utilisateur prsum sur ce poste et  cette date :
         (user,dConn) = suspect(poste, d)
      else:
         poste = items[2]              # on garde l'adresse IP si nom inconnu
         user =''
         dConn = 0.0
      print "%s/%s/%s  %s:%s:%s - Poste %s - Utilisateur %s ? :" % \
            (t[2],t[1],t[0],t[3],t[4],t[5], poste, user)
      print items[6]                         # l'item 6 est l'URL visite

# -- Programme principal : --

hotes_connus()          # Dictionnaire pour rsolution des adresses IP
log2smbDico()           # Dictionnaire journal de Samba

print'''
***********************************************************************
*** Recherches diverses dans les fichiers journaux de Samba & Squid ***
*** Auteur : Grard Swinnen, Verviers - version du %s       ***
*** Copyright : Licence GPL                                         ***
***********************************************************************''' % (version)

while 1:
   print '''
            *** Menu : ***
1. Occurrences d'une chane de caractres dans les URL visites
2. Liste des personnes ayant utilis un poste de travail
3. Idem, mais pour tous les postes de travail rpertoris
4. Postes de travail utiliss par une mme personne
5. Sortie'''
   print "              Votre choix --> ",
   ch = string.atoi(raw_input())
   print
   if ch ==1:
      postes_par_url()
   elif ch ==2:
      ut_par_poste()
   elif ch ==3:
      ut_tous_postes()
   elif ch ==4:
      postes_par_ut()
   else:
      break

